import scipy.ndimage as scind
def mfilter(imgarray,size=3, stat="mean"):
    """
    Mean, Median and Mode filter

    Parameters
    ----------
    imgarray : Image as ndArray.
        Satellite image as n-Dim numpy array.
    size : Filter size.
        All are square filter just add int value of one size like 3.
    stat : TYPE, optional
        "mean", "median", "mode". The default is "mean".

    Returns
    -------
    Filtered image according to the statiatics and kernel.

    """
    if stat=="mean":
        fimg=scind.uniform_filter(imgarray,size=size)
    elif stat=="median":
        fimg=scind.median_filter(imgarray,size=size)
    elif stat=="mode":
        fimg=scind.maximum_filter(imgarray,size=size)
    else:
        print("Please provide a valid statistical method: mean, median or mode")
    return fimg

def edgedetection(imgarray,mode="sobel",size=3):
    """
    Edge detection functions

    Parameters
    ----------
    imgarray : Image as ndArray.
        Satellite image as n-Dim numpy array..
    mode : type of filter , optional
        "sobel", "sobelH", "sobelV", "laplace", "prewitt", "prewittH","prewittV". The default is "sobel".
    size : Size of kernel for laplace, optional
        kernel size only for laplace edge detection. The default is 3.

    Returns
    -------
    Edge detection on satellite image.

    """
    if mode=="sobel":
        sobel_v=scind.filters.sobel(imgarray,axis=-1)
        sobel_h=scind.filters.sobel(imgarray,axis=1)
        out=(sobel_v+sobel_h)/2
    elif mode=="sobelH":
        out=scind.filters.sobel(imgarray,axis=1)
    elif mode=="sobelV":
        out=scind.filters.sobel(imgarray,axis=-1)
    elif mode=="laplace":
        out=scind.morphological_laplace(imgarray,size=size)
    elif mode=="prewitt":
        prewitt_v=scind.filters.prewitt(imgarray,axis=-1)
        prewitt_h=scind.filters.prewitt(imgarray,axis=1)
        out=(prewitt_v+prewitt_h)/2
    elif mode=="prewittH":
        out=scind.filters.prewitt(imgarray,axis=1)
    elif mode=="prewittV":
        out=scind.filters.prewitt(imgarray,axis=-1)
    else:
        print("Please provide a valid statistical method: mean, median or mode")
    return out
    


from .imageFrame import imagetoframe
import rasterio
from rasterio.mask import mask
import geopandas
import pandas
import rasterio

def extractbypolygon(imgpath,shppath,class_col=None):
    """
    Extract values from satellite image acoording to the given shapefile containing class information for classification.

    Parameters
    ----------
    imgpath : Path to satellite image.
    shppath : Path to ESRI shapefile.
    mode : mode of the operation no need to change, optional
        Don't change. The default is "r".
    class_col : Name of the column containing class values for classification.
        Need for arrange the table of output. The default is None.

    Returns
    -------
    tdataf : A table containing pixel values and corosponding class value.
        Need for train ML algo.

    """
    if class_col==None:
        print("Please enter the column name containing class information")
    else:
        img=rasterio.open(imgpath)
        meta=img.meta
        nod = meta['nodata']
        if not nod:
            nod = -999
        shp=geopandas.read_file(shppath)
        clsid=list(shp[class_col].unique())
        tdataf=pandas.DataFrame()
        for i in clsid:
            shpm=shp[shp[class_col]==i]
            maski,t=mask(img,shpm["geometry"],nodata=nod)
            df=imagetoframe(maski)
            df = df[(df.T != int(nod)).any()]
            df[class_col]=int(i)
            tdataf=tdataf.append(df)
            tdataf=tdataf.reset_index(drop=True)
        return tdataf



import rasterio
from rasterio.mask import mask
import geopandas
import numpy

def icrop(imgpath,shppath):
    """
    Crop satellite image using ESRI shapefile

    Parameters
    ----------
    imgpath : Path to satellite image.
    shppath : Path to ESRI shapefile.

    Returns
    -------
    mask1 : Satellite image as numpy ndarray
        Same extent of raw image.
    meta : Metadata of image.
        Needed at the time of save image.

    """
    shp=geopandas.read_file(shppath)
    img=rasterio.open(imgpath,"r")
    meta=img.meta
    mask1,a=mask(img, shp.envelope)
    return mask1,meta

def imask(imgpath,shppath,nodata=0):
    """
    Mask image according ESRI shapefile area.

    Parameters
    ----------
    imgpath : Path to satellite image.
    shppath : Path to ESRI shapefile.
    nodata : no data value, optional
        Value used for fill the outer shapefile area. The default is 0.

    Returns
    -------
    mask1 : Satellite image as numpy ndarray
        Same extent of raw image.
    meta : Metadata of image.
        Needed at the time of save image.

    """
    shp=geopandas.read_file(shppath)
    img=rasterio.open(imgpath,"r")
    meta=img.meta
    if nodata!=0:
        mask1,a=mask(img, shp["geometry"])
        mask1[mask1==0]=nodata
    else:
        mask1,a=mask(img, shp["geometry"])
    return mask1,meta

def layerStack(imglist):
    """
    Stack bands of satellite image.

    Parameters
    ----------
    imglist : list of image files.
        image file paths with directory and name.

    Returns
    -------
    ilist : Satellite image as mumpy ndarray
        Stacked in same order of input list.
    immeta : Updated metadata of the image with same file format.
        Needed at the time of save image.

    """
    ilist=[]
    for i in imglist:
        with rasterio.open(i,"r") as im:
            im2=im.read(1)
            immeta=im.meta
        ilist.append(im2)
    ilist=numpy.asarray(ilist)
    immeta["count"]=ilist.shape[0]
    return ilist,immeta

def ndi(band_1,band_2,nodata=0,replace=True):
    """
    Normalized Difference Index. e.g. NDVI.

    Parameters
    ----------
    band_1 : First band of the index
        e.g. NIR band for NDVI.
    band_2 : Second band of the index
        e.g. RED band for NDVI.
    nodata : nodata value if any present in the input image, optional
        The default is 0.
    replace : Change the nodata value to nan, optional
        If True change the nodata value to np.nan. The default is True.

    Returns
    -------
    index : Normalized Difference Index.

    """
    band_1[band_1==nodata]=numpy.nan
    band_2[band_2==nodata]=numpy.nan
    index=(band_1-band_2)/(band_1+band_2)
    if replace == False:
        index=numpy.nan_to_num(index,nan=nodata)
    return index


import rasterio as rio
import numpy as np
import pandas as pd

def ts_stackR(imglist,replace_nodata=True):
    """
    Stack multi-time image list to stacked numpy array

    Parameters
    ----------
    imglist : list of image files.
        image file paths with directory and name.
    replace_nodata : True/False, optional
        convert nodata value to nan if present in the image metadata. The default is True.

    Returns
    -------
    ilist : stacked numpy array of images.
        where 3rd dim represent different time.
    meta : Updated metadata with new bands.

    """
    ilist=[]
    for i in imglist:
        with rio.open(i,"r") as sr:
            img=sr.read()
            meta=sr.meta
        ilist.append(img)
    ilist=np.vstack(ilist)
    meta['count']=ilist.shape[0]
    if replace_nodata==True:
        if meta['nodata'] != None:
            ilist[ilist==meta['nodata']]=np.nan
    return ilist,meta

def ts_frameR(imgarray,startD=None,endD=None,freq="D"):
    """
    Convert time series image array to time series frame. Where band represent Day, Month or Year.

    Parameters
    ----------
    imgarray : image as numpy ndarray.
    startD : Stating data of the image.
        In format of 'YYYY-MM-DD','YYYY' or 'DD'. The default is None.
    endD : End data of the image.
        In format of 'YYYY-MM-DD','YYYY' or 'DD'. The default is None.
    freq : time frequency.
        same as pandas timedate, Visit https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#timeseries-offset-aliases. The default is "D".

    Returns
    -------
    iframe : Dataframe where index represent time and column represent pixels.
        DESCRIPTION.

    """
    if startD==None or endD==None:
        print("Please enter avalid Date.")
        return
    iarray=[]
    for i in range(imgarray.shape[0]):
        iarray.append(imgarray[i].flatten())
    iarray=np.asarray(iarray)
    iframe=pd.DataFrame(iarray,index=pd.date_range(startD,endD,freq=freq))
    return iframe

def ts_arrayR(imgframe,width=None,height=None):
    """
    Convert time series image frame to image array

    Parameters
    ----------
    imgframe : Stuctured image data.
        Time series image frame.
    width : Width of the image array.
    height : Height of the image array.

    Returns
    -------
    imgr : Image as numpy array.
        bands represent time.

    """
    if width==None or height==None:
        print("Please enter image width and height")
        return
    iarray=imgframe.to_numpy()
    imgr=[]
    if len(imgframe.columns)==1:
        imgr=iarray[0].reshape(height,width)
    else:
        for i in range(iarray.shape[0]):
            imgr.append(iarray[i].reshape(height,width))
    imgr=np.asarray(imgr)
    return imgr



import setuptools
with open("README.md") as r:
    readme=r.read()

for line in open("satiopst\__init__.py").read().splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            var=str(line.split(delim)[1])

setuptools.setup(
    name="SatIOpsT",
    version=var,
    author="GeoEspacioTech Pvt. Ltd., Subhadip Datta, Soumyadeep Dutta",
    author_email="geoespaciotech@gmail.com",
    description="Satellite Image Operations Toolbox",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/SubhadipDatta/SatIOpsT/wiki",
    packages=setuptools.find_packages(exclude=('tests', 'docs')),
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "geopandas",
        "rasterio",
        "scipy",
        "pandas",
        "numpy",
    ],
    include_package_data=True,
)

